/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2004 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

Application
    icoTopoFoamTJunction

Description
    Select faces for face zones and add them to the mesh.  Add an
    attach-detach modifier; set up for the T-junction case.

Author and Copyright notice:
    Hrvoje Jasak.  All rights reserved.

\*---------------------------------------------------------------------------*/


#include "fvCFD.H"
#include "attachDetach.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

int main(int argc, char *argv[])
{

#   include "setRootCase.H"

#   include "createTime.H"
#   include "createMesh.H"

    const vectorField& faceCentres = mesh.faceCentres();
    const vectorField& faceAreas = mesh.faceAreas();

    labelList zoneFaces(faceCentres.size());
    boolList zoneFlip(faceCentres.size(), false);

    label nZoneFaces = 0;

    // Add face zone

    forAll (faceCentres, faceI)
    {
        if
        (
            faceCentres[faceI].x() > 3
         && faceCentres[faceI].x() < 4
         && faceCentres[faceI].y() < 0.01
         && faceCentres[faceI].y() > -0.01
         && mag(faceAreas[faceI].y()) > SMALL
        )
        {
            zoneFaces[nZoneFaces] = faceI;

            if (faceAreas[faceI].y() < VSMALL)
            {
                zoneFlip[nZoneFaces] = true;
            }

            nZoneFaces++;
        }
    }

    zoneFaces.setSize(nZoneFaces);
    zoneFlip.setSize(nZoneFaces);

    Info << "zoneFaces: " << zoneFaces << nl
         << "zone flip: " << zoneFlip << endl;

    List<pointZone*> pz(0);
    List<faceZone*> fz(1);
    List<cellZone*> cz(0);

    fz[0] =
        new faceZone
        (
            "membraneFaces",
            zoneFaces,
            zoneFlip,
            0,
            mesh.faceZones()
        );

    mesh.addZones(pz, fz, cz);

    // Add modifier
    List<polyMeshModifier*> tm(1);

    // Create list of trigger times
    scalarField triggerTimes(5);
    triggerTimes[0] = 10.0;
    triggerTimes[1] = 20.0;
    triggerTimes[2] = 30.0;
    triggerTimes[3] = 40.0;
    triggerTimes[4] = 50.0;

    tm[0] =
        new attachDetach
        (
            "TjunctionGate",
            0,
            mesh,
            "membraneFaces",
            "gateTop",
            "gateBottom",
            triggerTimes
        );


    mesh.addTopologyModifiers(tm);

    mesh.write();

    Info<< "End\n" << endl;

    return(0);
}


// ************************************************************************* //
